function varargout=plotiso(self,u,varargin)
% FUNCTION plotiso 
%   Plots isolines from scalar data values on the siMesh object.
%
% USAGE
%   Th.plotiso(u) or plotiso(Th,u) 
%     where Th is a siMesh object and u is a vector of length Th.nq which 
%     contains the scalar data values on each vertices of the siMesh object. 
%     This function only acts on the 2-simplex mesh elements of the siMesh 
%     object.
%
%   Th.plotiso(u,key,value,...) allows additional key/value pairs to be 
%     used when displaying u. The key strings could be
%
%     'labels' to specify the labels of the 2-simplex elementary meshes. 
%              Default [] i.e all.
%     'niso'   number of isolines to draw. The associated isovalues are
%              taken as linspace(min(u),max(u),niso).Default value is 10.
%     'isorange' to manualy specify all the isovalues.
%     'color'  to specify a same color for all the isolines.
%     'plan'   (only in dimension 2), if true display is done in x0y plane,
%              otherwise the z-axe is the scalar data values. Default is false.
%     'isocolorbar' if true display a colorbar.Default is false.
%     'format' to specify the format of the isovalues print in the colorbar.
%              Default is '%g'
%
%   or any of the folowing options. Same key/pairs as
%
%     - plot3   function for d=2 and dim=3
%     - plot    function for d=2 and dim=2 with 'plan'==true
%     - plot3   function for d=2 and dim=2 with 'plan'==false
%
%   varargout=Th.plotiso(...)
%     This function accepts until 4 output arguments :
%     - 1st output is the colors of the isolines
%     - 2nd output is the isovalues of the isolines
%     - 3th output is the handle of the colobar iso.
%     - 4th output is all the handles of the isolines as an 2D-array of 
%       dimension N-by-niso, where N is the number of 2-simplex elementary
%       meshes where isolines are drawn.
% 
% SAMPLES
%   Th=fc_simesh.HyperCube(3,10);
%   U=Th.eval(@(x,y,z) cos(2*x-y+z).*sin(x+y-3*z));
%   figure(1)
%   Th.plotiso(U)
%   figure(2)
%   Th.plotiso(U,'labels',[1,2,5],'color','w')
%
%    <COPYRIGHT>
  p = inputParser; 
  p.KeepUnmatched=true; 
  p.PartialMatching=false;
  p.addParamValue('labels',[]); % select labels to draw
  p.addParamValue('niso',10,@isscalar);
  p.addParamValue('isorange',[]);
  p.addParamValue('isocolorbar',false,@islogical);
  p.addParamValue('plan',false);
  p.addParamValue('color',[]); % unique color
  p.addParamValue('format','%g',@ischar);
  %p.addParamValue('FontSize',8);
  p.addParamValue('mouse',true,@islogical);
  p.parse(varargin{:});
  R=p.Results;
  varargin=fc_tools.utils.deleteCellOptions(varargin,p.Parameters);
  if isempty(R.labels), R.labels=self.sThlab(self.sThsimp==2);end
  if isempty(R.labels)
  %  warning('No label found for %d-simplex sub-meshes',2);
    return;
  end
  umin=min(u);umax=max(u);
  if isempty(R.isorange)
    R.isorange=linspace(umin,umax,R.niso);
  else
    R.isorange=sort(R.isorange);
  end
  R.niso=length(R.isorange); 
  if isempty(R.color)
    cmap=colormap();%R.colormap);
    umin=min(umin,R.isorange(1)); % add for partitioned meshes
    umax=max(umax,R.isorange(end));
    t=(R.isorange-umin)/(umax-umin);
    colors = fc_tools.graphics.RGB(t,cmap);
  else
    if ischar(R.color), R.color = fc_tools.graphics.colorstr2rgb(R.color);end
    colors=ones(R.niso,1)*R.color;
  end
  
  LB=self.find(2);
  PrevHold=fc_tools.graphics.SetHoldOn(true);
  h=zeros(length(R.labels),R.niso);
  set(gcf(),'visible','off')
  for l=1:length(R.labels)
    [I,K]=ismember(R.labels(l),self.sThlab(LB));
    if K~=0
      Ilab=LB(K);
      [~,~,h(l,:)]=self.sTh{Ilab}.plotiso(u,'niso',R.niso,'isorange',R.isorange,'plan',R.plan, ...
                                  'color',colors,varargin{:});
    end
  end
  if ( (self.dim==3) && (length(axis())==4) ), view(3);end
  cax=[];
  if R.isocolorbar
    cax=fc_tools.graphics.colorbarIso(colors,R.isorange,R.format);
  end
  if R.mouse && ~fc_tools.comp.isOctave() % Only Matlab
    datacursormode on
    dcm = datacursormode(gcf);
    set(dcm,'UpdateFcn',@fc_simesh.showmeshinfo)
  end
  if nargout>=1, varargout{1}=colors;end
  if nargout>=2, varargout{2}=R.isorange;end
  if nargout>=3, varargout{3}=cax;end
  if nargout==4, varargout{4}=h;end
  fc_tools.graphics.RestoreHold(PrevHold)
  set(gcf(),'visible','on')
end
